unit Winsaver;

interface

uses SysUtils, WinProcs, Forms, Classes, Dialogs;

type

TWinSaver = class(TComponent)
 private
	FIniFileName: string;
        FIsLocal: Boolean;
	FIniSect: string;
        FIsSaveSize: Boolean;
        FIsSaveLocation: Boolean;
        FIsSaveState: Boolean;
        procedure SaveWindow(TheForm: TForm);
        procedure SaveChildren(TheForm: TForm);
        procedure OpenIniFile;
	procedure Save(Sender: TObject; var Action: TCloseAction);
	procedure Restore(Sender: TObject);
        procedure RestoreWindow(TheForm: TForm);
        procedure RestoreChildren(TheForm: TForm);
 public
	constructor Create(AOwner: TComponent); override;
 published
        property IniFileName: string read FIniFileName write FIniFileName;
        property LocalDir: Boolean read FIsLocal write FIsLocal default False;
        property IniSection: string read FIniSect write FIniSect;
        property SaveSize: Boolean read FIsSaveSize write FIsSaveSize default True;
        property SaveLocation: Boolean read FIsSaveLocation write FIsSaveLocation default True;
        property SaveState: Boolean read FIsSaveState write FIsSaveState default True;
end;

procedure register;

implementation

uses IniFiles;

var
   Ini: TIniFile;

constructor TWinSaver.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);

        { initial property value }

        FIsLocal := False;
        FIsSaveSize := True;
        FIsSaveLocation := True;
        FIsSaveState := True;
        FIniFileName := ChangeFileExt(ExtractFileName(AOwner.Name), '.INI');
        FIniSect := 'WinSaver';

        (AOwner as TForm).OnCreate := Restore;
        (AOwner as TForm).OnClose  := Save;
end;

procedure TWinSaver.RestoreWindow(TheForm: TForm);
begin
     with TheForm do begin
          if FIsSaveSize then begin
             Width := Ini.ReadInteger(FIniSect, ClassName + '_Width', Width);
             Height := Ini.ReadInteger(FIniSect, ClassName + '_Height', Height);
          end;

          if FIsSaveLocation then begin
             Top := Ini.ReadInteger(FIniSect, ClassName + '_Top', Top);
             Left := Ini.ReadInteger(FIniSect, ClassName + '_Left', Left);
          end;

          if FIsSaveState then
             case Ini.ReadInteger(FIniSect, ClassName + '_WindowState', 2) of
                  1: WindowState := wsMinimized;
                  2: WindowState := wsNormal;
                  3: WIndowState := wsMaximized;
             end;
     end;
end;


procedure TWinSaver.RestoreChildren(TheForm: TForm);
var i: integer;
begin
     for i := 0 to TheForm.MDIChildCount-1 do
         RestoreWindow(TheForm.MDIChildren[i]);
end;


procedure TWinSaver.Restore(Sender: TObject);
var i: Integer;
begin
        OpenIniFile;
	try
	   for i := 0 to Application.ComponentCount-1 do
               if (Application.Components[i] is TForm) then begin
                  RestoreWindow(TForm(Application.Components[i]));
                  if TForm(Application.Components[i]).FormStyle = fsMDIForm then
                     RestoreChildren(TForm(Application.Components[i]));
               end;
	finally
           Ini.free;
	end;
end;


{procedure TWinSaver.Save(Sender: TObject; var Action: TCloseAction);}
procedure twinsaver.save;
var i: Integer;
begin
        OpenIniFile;
	try
	   for i := 0 to Application.ComponentCount-1 do
               if (Application.Components[i] is TForm) then begin
                  SaveWindow(TForm(Application.Components[i]));
                  if TForm(Application.Components[i]).FormStyle = fsMDIForm then
                     SaveChildren(TForm(Application.Components[i]));
               end;
	finally
           Ini.free;
	end;
end;

procedure TWinSaver.SaveWindow(TheForm: TForm);
begin
     with TheForm do begin
          if FIsSaveSize then begin
             Ini.WriteInteger(FIniSect, ClassName + '_Width', Width);
             Ini.WriteInteger(FIniSect, ClassName + '_Height', Height);
          end;

          if FIsSaveLocation then begin
             Ini.WriteInteger(FIniSect, ClassName + '_Top', Top);
             Ini.WriteInteger(FIniSect, ClassName + '_Left', Left);
          end;

          if FIsSaveState then
             case TheForm.WindowState of
                  wsMinimized: Ini.WriteInteger(FIniSect, ClassName + '_WindowState', 1);
                  wsNormal:    Ini.WriteInteger(FIniSect, ClassName + '_WindowState', 2);
                  wsMaximized: Ini.WriteInteger(FIniSect, ClassName + '_WindowState', 3);
             end;
     end;
end;

procedure TWinSaver.SaveChildren(TheForm: TForm);
var i: integer;
begin
     for i := 0 to TheForm.MDIChildCount-1 do
         SaveWindow(TheForm.MDIChildren[i]);
end;

procedure TWinSaver.OpenIniFile;
var
   buf: PChar;
   RealFileName: string;
begin
        if FIniFileName = '' then     { if IniFile property is empty , set to appname.ini}
           FIniFileName := ChangeFileExt(ExtractFileName(Application.ExeName), '.INI')
        else        { change to '.ini' extesion }
           FIniFileName := ChangeFileExt(ExtractFileName(FIniFileName), '.INI');

        GetMem(buf,256);
        try
           if FIsLocal = True then
              GetCurrentDirectory(256,buf)
           else
              GetWindowsDirectory(buf,256);

           RealFileName := StrPas(buf) + '\' + FIniFileName;
        finally
           FreeMem(buf,256);
        end;

        if FIniSect = '' then        {if IniSection property is empty, set to [WinSaver]}
           FIniSect := 'WinSaver';

       	Ini := TIniFile.Create(RealFileName);
end;

procedure register;
begin
     RegisterComponents('system',[TWinSaver]);
end;

end.
